from typing import List

from res.enkf import EnKFMain
from res.enkf.enums.enkf_obs_impl_type_enum import EnkfObservationImplementationType
from res.enkf.enums.enkf_var_type_enum import EnkfVarType


class ErtSummary:
    def __init__(self, ert: EnKFMain):
        self.ert = ert

    def getForwardModels(self) -> List[str]:
        forward_model = self.ert.getModelConfig().getForwardModel()
        return list(forward_model.joblist())

    def getParameters(self) -> List[str]:
        parameters = self.ert.ensembleConfig().getKeylistFromVarType(
            EnkfVarType.PARAMETER
        )
        return sorted(parameters, key=lambda k: k.lower())

    def getObservations(self) -> List[str]:
        gen_obs = self.ert.getObservations().getTypedKeylist(
            EnkfObservationImplementationType.GEN_OBS
        )

        summary_obs = self.ert.getObservations().getTypedKeylist(
            EnkfObservationImplementationType.SUMMARY_OBS
        )

        keys = []
        summary_keys_count = {}
        summary_keys = []
        for key in summary_obs:
            data_key = self.ert.getObservations()[key].getDataKey()

            if not data_key in summary_keys_count:
                summary_keys_count[data_key] = 1
                summary_keys.append(data_key)
            else:
                summary_keys_count[data_key] += 1

            if key == data_key:
                keys.append(key)
            else:
                keys.append(f"{key} [{data_key}]")

        obs_keys = list(gen_obs) + summary_keys
        return sorted(obs_keys, key=lambda k: k.lower())
